@@ -1,4 +1,4 @@
-import tempfile
+import multiprocessing
 import os
 import json
 import threading
@@ -6,18 +6,19 @@ import sys
 import time
 import gc
 import pathlib
-import shutil
-import multiprocessing
 import subprocess
 import urllib.request
-
+import concurrent.futures
+import importlib
 import scrapy.crawler
+
 from .RashScrappers.RashScrappers.spiders import *
 
 __all__ = [
     "JsonHandler",
     "TempHandler",
-    "Launcher"
+    "Launcher",
+    "Processes"
 ]
 
 URL = "https://github.com/RahulARanger/RashSetup"
@@ -59,7 +60,7 @@ LOCK SAUCE:
 
 
 class Launcher:
-    def __init__(self, pwd):
+    def __init__(self, pwd, manager: concurrent.futures.ThreadPoolExecutor):
         self.pwd = pathlib.Path(pwd)
         self.pwd = self.pwd.parent if self.pwd.is_file() else self.pwd
 
@@ -71,18 +72,17 @@ class Launcher:
 
         None if self.test() else self._notify()
 
-        read_thread = threading.Thread(target=self.read_thread)
-        read_thread.setName("Global Mutex Thread")
-        read_thread.setDaemon(True)
-        write_thread = threading.Thread(target=self.write_thread)
-        write_thread.setName("Max Mutex Thread")
-        write_thread.setDaemon(True)
-
         self.workers = threading.Lock(), threading.Lock()
-        self.remainder = None
 
-        read_thread.start()
-        write_thread.start()
+        manager.submit(
+            self.read_thread
+        )
+
+        manager.submit(
+            self.write_thread
+        )
+
+        self.remainder = None
 
     def _notify(self):
         self.max_mutex.write_text("1")
@@ -149,59 +149,24 @@ class Launcher:
             _.release()
 
 
-def export_rash(current_path):
-    if not os.path.isdir(current_path):
-        raise NotADirectoryError(f"{current_path} is not a directory")
-
-
-def pip_install(*packages):
-    for _ in packages:
-        subprocess.run([
-            sys.executable, "-m", "pip", "install", _
-        ])
-
-
-def is_rash():
-    return sys.executable.endswith("Rash.exe")
-
-
-def check_settings(path):
-    return "settings.json" in os.listdir(path)
-
-
-gc.collect()
-
-
 class PluginManager:
-    def __init__(self):
+    def __init__(self, logger=None):
         self.file = JsonHandler(os.path.join(
             os.path.dirname(__file__), "settings.json"
         ))
 
-        self.modules = os.path.join(
-            os.path.dirname(__file__), "Modules"
-        )
-
-        self.general = os.path.join(
-            self.modules, "General"
-        )
-
-        self.user = os.path.join(
-            self.modules, "User"
-        )
-
+        self.logger = logger
         self.employ()
 
-    def is_useful(self):
-        return os.path.exists(str(self.file)) and all(
-            os.path.exists(__)
-            for _ in self.file.load()["general"]["modules"]
-            for __ in os.path.join(self.user, _)
+    def is_useful(self, cache=None):
+        cache = cache if cache else self.file.load()
+        return all(
+            importlib.util.find_spec(_) for _ in cache["general"]["modules"]
         )
 
     def employ(self):
-        reactor = self.scrape_settings() if not os.path.exists(
-            str(self.file)) else None if self.is_useful() else self.setup(refresh=True)
+        reactor = None if os.path.exists(str(self.file)) else self.scrape_settings()
+        reactor = self.setup(reactor) if reactor or not self.is_useful() else None
         reactor.start() if reactor else None
 
     def scrape_settings(self):
@@ -210,72 +175,122 @@ class PluginManager:
         settings = process.crawl(
             SettingsSpider,
             url=URL,
+            path=None,
             pipe=self.update_settings
         )
 
         settings.addCallback(
-            lambda _: self.setup(process, True)
+            lambda _: self.setup(process)
         )
 
-        process.start()
-
-    def setup(
-            self,
-            process=None,
-            refresh=False
-    ):
-
-        shutil.rmtree(self.modules) if refresh and os.path.exists(self.modules) else None
-
-        process = process if process else scrapy.crawler.CrawlerProcess()
-        self.ensure_paths()
-
-        general = self.file.load()['general']['modules']
-
-        for _ in general:
-            module = os.path.join(
-                self.user, _
-            )
-
-            if os.path.exists(module):
-                continue
-
-            process.crawl(
-                RepoSpider,
-                url=general[_],
-                pipe=self.sample_pipe,
-                save=os.path.join(self.general, _)
-            )
-
         return process
 
-    def update_settings(self, url):
+    def update_settings(self, path, url):
+        temp = JsonHandler(os.path.join(
+            path, "settings.json")
+        ) if path else self.file
+
         with urllib.request.urlopen(url) as o_json:
-            parsed = self.file(
+            parsed = temp(
                 o_json.read()
             )
 
-        user = self.file.load()["user"] if self.is_useful() else False
-
         if parsed is None:
             raise TypeError(f"Not able to parse {url}")
 
-        parsed["user"].update(user) if user else None
+        parsed["user"].update(temp.load()["user"]) if os.path.exists(temp.file) else None
 
         self.file.dump(parsed)
 
-    def sample_pipe(self, info):
-        print(info)
+    def setup(
+            self,
+            process=None
+    ):
+        process = process if process else scrapy.crawler.CrawlerProcess()
 
-    def ensure_paths(self):
-        None if os.path.exists(self.modules) else os.mkdir(self.modules)
-        None if os.path.exists(self.general) else os.mkdir(self.general)
-        None if os.path.exists(self.user) else os.mkdir(self.user)
+        general = self.file.load()['general']['modules']
+
+        for _ in general:
+            if importlib.util.find_spec(_):
+                continue
+
+            self.install_module(
+                process, _, general[_]
+            )
+
+        return process
+
+    def install_module(self, process, name, url):
+        process.crawl(
+            RepoSpider,
+            url=url,
+            pipe=lambda saved: self.install_settings(saved),
+            name=name
+        )
+
+    def install_settings(self, saved):
+        # assumes that ur module has settings.json inside it
+        settings = os.path.join(saved, "settings.json")
+        setup = os.path.join(saved, "setup.py")
+
+        if not os.path.exists(settings):
+            raise FileNotFoundError(f"Settings File not found for {settings}")
+
+        Processes.setup_tools(setup)
+        Processes.pip_install(saved)
+
+    def uninstall_module(self, name):
+        importlib.util.find_spec(name)  # TODO: a simple pip uninstall may not delete everything
+        # TODO: this affects when reinstalling the package
+
+        subprocess.run([
+            sys.executable,
+            "-m",
+            "pip",
+            "uninstall",
+            name,
+            "-y"
+        ])
 
 
-def ParentProcess():
-    temp = PluginManager()
+class Processes:
+    @staticmethod
+    def pip_install(package, *args):
+        subprocess.run([
+            sys.executable, "-m", 'pip', 'install', package, *args
+        ])
+
+    @staticmethod
+    def setup_tools(setup_file):
+        subprocess.run([
+            sys.executable, setup_file, "sdist bdist"
+        ])
+
+    @staticmethod
+    def pip_uninstall(package):
+        subprocess.run([
+            sys.executable, "-m", "pip", "uninstall", package, "-y"
+        ])
+
+    @staticmethod
+    def rash_process():
+        try:
+            temp = PluginManager()
+
+        except Exception as error:
+            print("Failed to Setup Rash! because of", error)
+
+            while True:
+                pass
+
+    @staticmethod
+    def search(url, queue: multiprocessing.Queue):
+        def results(path, raw):
+            queue.put(raw)
+
+        process = scrapy.crawler.CrawlerProcess()
+        process.crawl(SettingsSpider, url=url, pipe=results, path=None)
+        process.start()
 
 
-def ChildProcess():
-    pass
+gc.collect()
